@@ -5,3 +5,8 @@
         swe_set_ephe_path(system.file("ephemeris", package = "swephR", mustWork = TRUE))
     }
 }
+
+.onUnload <- function (libpath) {
+    swe_close()
+    library.dynam.unload("swephR", libpath)
+}
