@@ -63,7 +63,7 @@
 
 /*
   swe_date_conversion():
-  This function converts some date+time input {d,m,y,uttime}
+  This function converts some date+time input {y,m,d,uttime}
   into the Julian day number tjd.
   The function checks that the input is a legal combination
   of dates; for illegal dates like 32 January 1993 it returns SE_ERR
@@ -110,7 +110,7 @@ int CALL_CONV swe_date_conversion(int y,
   } else {
     return SE_ERR;
   }
-}	/* end date_conversion */
+}
 
 /*************** swe_julday ********************************************
  * This function returns the absolute Julian day number (JD)
@@ -148,16 +148,13 @@ int CALL_CONV swe_date_conversion(int y,
 
  Original author: Marc Pottenger, Los Angeles.
  with bug fix for year < -4711   15-aug-88 by Alois Treindl
- (The parameter sequence m,d,y still indicates the US origin,
-  be careful because the similar function date_conversion() uses
-  other parameter sequence and also Astrodienst relative juldate.)
  
  References: Oliver Montenbruck, Grundlagen der Ephemeridenrechnung,
              Verlag Sterne und Weltraum (1987), p.49 ff
  
  related functions: swe_revjul() reverse Julian day number: compute the
   			       calendar date from a given JD
-	            date_conversion() includes test for legal date values
+	            swe_date_conversion() includes test for legal date values
 		    and notifies errors like 32 January.
  ****************************************************************/
 
